import demo_buffer
import thread
import time
import sys
import math
from math import sin, cos

from mosaic_trans import buf, blocks, getblock, getColouredCharacter
import pygametext

import pygame
pygame.mixer.pre_init(44100)
pygame.init()
pygame.mixer.init(44100)
surface = pygame.Surface((80,70))
bg = pygame.Surface((80,70))
bg.fill((0,0,0))
# Buffer size: 42 x 24

music = pygame.mixer.Sound("tekstitv3n.wav")

DEBUG = True

# Pygame colors
BLACK = (0,0,0)
WHITE = (255,255,255)
RED = (255,0,0)
GREEN = (0,255,0)
BLUE = (0,0,255)

ham84 = [0x15, 0x02, 0x49, 0x5E, 0x64, 0x73, 0x38, 0x2F, 0xD0, 0xC7, 0x8C, 0x9B, 0xA1, 0xB6, 0xFD, 0xEA ]

def setBit(i, offset):
	mask = 1 << offset
	return(i|mask)   

def parityOf(i):
	parity = 0
	while(i):
		parity = ~parity
		i = i & (i - 1)
	return(parity)

def clearBit(i, offset):
	mask = ~(1 << offset)
	return (i & mask)

def toBuffer(x, y, inp, correction=True):
    intarr = []
    if(correction == True):
        if(type(inp) == str):
            for c in inp:
                intarr.append(ord(c))
        elif(type(inp) == list):
            intarr = inp
        else:
            intarr[0] = inp

    if(correction):
        i = 0
        for c in intarr:
            c = clearBit(c,7)
            p = parityOf(c)
            if(p == 0):
                c = setBit(c,7)
            intarr[i] = c
            i += 1
    for c in intarr:
        try:
            demo_buffer.setBuffer(y, x, c)
            x += 1
        except:
            sys.stderr.write("Out-of-bounds! Exiting\n")
            sys.stderr.write("X: %s Y: %s" % (x,y))
            exit(1)

def setHeader():
    headtext = " AssemblyTV'76 - 05.08.1976     "
    toBuffer(10,0, headtext)

def drawBitmap(skipping = True, holdGfx = True):
    global buf
    blank()
    skipX = False
    for y in range(1,23):
        c = None
        skipX = False
        if(holdGfx == True):
            toBuffer(2,y,chr(0x1e)) # Hold graphics
        for x in range(4,38):
            if(skipX == True):
                skipX = False
                continue
            charPair = getColouredCharacter(x,y,c)
            if(len(charPair) > 1):
                toBuffer(x-1,y,charPair)
                c = ord(charPair[0])
                if(skipping == True):
                    skipX = True
            else:
                toBuffer(x,y,charPair)

def drawHexmapFromFile(fname):
    f = file(fname, "r")
    lines = f.readlines()
    while True:
        x = 0
        y = 0
        for l in lines:
            l = l.strip()
            chars = l.split(' ')
            for c in chars:
                h = int(c,16)
                toBuffer(x,y,[h])
                x += 1
            y += 1
            x = 0

def dataFromFile(fname):
    f = file(fname, "rb")
    lines = f.readlines()
    y = 1
    for l in lines:
        x = 2
        l = l[:-1]
        l2 = []
        if(y == 24):
            break
        toBuffer(2, y, l)
        y += 1

def starfield():
    start = time.time()
    global buf
    passes = 0
    MAXPARTICLES = 40
    particles = []
    speed = 1.5
    WIDTH = len(buf[0])
    HEIGHT = len(buf)
    while (time.time()-start < 15):
        blankBitmap()
        if(len(particles) < MAXPARTICLES):
            particles.append([WIDTH/2,HEIGHT/2, len(particles)*passes % (2*math.pi)]) # [x,y,angle in radians]
        i = 0
        while i < len(particles):
            p = particles[i]
            p[0] = p[0] + cos(p[2])*speed
            p[1] = p[1] + sin(p[2])*speed
            try:
                buf[int(p[1])][int(p[0])] = "7"
                buf[int(p[1])+10][int(p[0])+10] = "3"
                buf[int(p[1])-5][int(p[0])-5] = "2"
                buf[int(p[1])-13][int(p[0])-13] = "6"
                particles[i] = p
            except:
                p = [WIDTH/2, HEIGHT/2, (len(particles)-1)*passes % (2*math.pi)]
                particles[i] = p
            i += 1
        drawBitmap(holdGfx = False)
        demo_buffer.commit()
        time.sleep(0.1444)
        passes += 1

def sinewave(passes):
    global buf
    i = 2
    blankBitmap()
    for x in range(4,80):
        if(i == 8):
            i = 2
        y = int(30*math.sin(((x+passes)/40.0)*(math.pi*2)))+36
        buf[y][x] = str(i)
        y = int(30*math.cos(((x+passes)/40.0)*(math.pi*2)))+36
        buf[y][x] = str(i)
        i += 1
    drawBitmap(True, True)

def blankBitmap():
    global buf
    for y in range(0,len(buf)-1):
        for x in range(0, len(buf[0])-1):
            buf[y][x] = "0"

def blank():
    for y in range(1,24):
        for x in range(2,42):
            toBuffer(x, y, [0x20])

def sdlsine(t):
    global surface
    global bg
    global buf
    surface.blit(bg, (0,0))
    w = surface.get_width()
    h = surface.get_height()
    
    for x in range(0, w):
        y = int((10 /0.5)*math.sin(((x+t)/(h/2)*(math.pi*2)))+(h/2))
        pygame.draw.circle(surface, WHITE, (x,y), 3)
    buf = pygametext.surfaceToTelemap(surface, buf)
    drawBitmap(True)

def vslider(t, start, end):
	fullblock = 0x7f
	x = int(39*(t-start)/end)+2
        blank()
        for y in range(3,23):
                toBuffer(x, y, [fullblock])

def hslider(t, start, end):
	fullblock = 0x7f
	row = [0x17,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock,fullblock] 
	y = int(22*(t-start)/end)+2
        blank()
        toBuffer(2,y, row)

def heraser(elapsed, start, total, skipBitmap = False):
    global buf
    if(elapsed < start):
        return
    t = elapsed-start
    print >> sys.stderr, "T:", t
    maxrow = int(len(buf)*((t/total)))
    print >> sys.stderr, "Maxrow:",maxrow
    if(skipBitmap == False):
        for y in range(0,maxrow):
            for x in range(0,len(buf[y])):
                buf[y][x] = '0'
        drawBitmap()
    else:
        for y in range(0,maxrow/3):
            toBuffer(2, y, ' '*38)

def hreverser(elapsed, start, total, skipBitmap = False):
    global buf
    if(elapsed < start):
        return
    t = elapsed-start
    print >> sys.stderr, "T:", t
    maxrow = int(len(buf)*(1-(t/total)))
    print >> sys.stderr, "Maxrow:",maxrow
    if(skipBitmap == False):
        for y in range(0,maxrow):
            for x in range(0,len(buf[y])):
                buf[y][x] = '0'
        drawBitmap()
    else:
        for y in range(0,maxrow/3):
            toBuffer(2, y, ' '*39)



if __name__ == '__main__':
    setHeader()
    dataFromFile("logo.hex")
    demo_buffer.commit()
    thread.start_new_thread(demo_buffer.print_buffer, (0,))

    time.sleep(5) # Logo-slide
    start = time.time()
    music.play()
    while True:
        elapsed = time.time()-start
        frameStart = time.time()
        print >> sys.stderr, "Elapsed:", elapsed
        if(elapsed < 5):
            dataFromFile("amiga.hex")
        elif(elapsed < 10):
            dataFromFile("teletext.hex")
        elif(elapsed < 15):
            dataFromFile("info.hex")
        elif(elapsed < 24):
            vslider(elapsed, 15, 9)
        elif(elapsed < 30):
            hslider(elapsed, 24, 6)
        elif(elapsed < 35):
            dataFromFile("delfiini.hex")
        elif(elapsed < 40):
            sinewave(elapsed)
            heraser(elapsed, 35, 5)
        elif(elapsed < 55):
            sdlsine(elapsed)
            hreverser(elapsed, 40, 5)
            heraser(elapsed, 52, 3)
        elif(elapsed < 65):
            dataFromFile("greetz.hex")
        elif(elapsed < 75): # In practice 15sec, avoid rerun
            starfield()
        elif(elapsed < 90):
            dataFromFile("credits.hex")
        else:
            blank()
            demo_buffer.commit()
            time.sleep(5)
            exit(0)
        demo_buffer.commit()
        frameStop = time.time()
        sleepTime = 0.1444-(frameStop-frameStart)
        print >> sys.stderr, "Sleep: ",sleepTime
        if(sleepTime > 0):
            time.sleep(sleepTime)
        else:
            time.sleep(0.07)
